@Version 2, March, 2009.
These routines can be used by and distributed for non-profit academic purposes
without any royalty except that the users must cite:
Perron and Yabu (2009): "Estimating Deterministic Trends with an Integrated or
Stationary Noise Component," forthcoming in Journal of Econometrics.
Perron and Yabu (2009): "Testing for Shifts in Trend with an Integrated or Stationary 
Noise Component," forthcoming in Journal of Business and Economic Statistics.
for any other commercial use and/or comments,  please contact 
Pierre Perron at perron at bu.edu.
Even though we tried to make this program error-free we cannot be held
responsible for any consequences that could result from remaining errors.
Copyright, Pierre Perron and Tomoyoshi Yabu (2009).@

new;
format /ld 6,4;


output file = results.out reset; @select your output file@

load y[117,1]=canada.asc;
/*Annual GDP for the Canada from 1870 to 1986*/

model=3;  
/*Choose a model from the following:
Model=1: Y{t}=a0+a1*DU+b0*t+e{t} where DU=1(t>TB)
Model=2: Y{t}=a0+b0*t+b1*DT+e{t} where DT=1(t>TB)*(t-TB)
Model=3: Y{t}=a0+a1*DU+b0*t+b1*DT+e{t} where DU=1(t>TB) and DT=1(t>TB)*(t-TB)*/

criteria=2; @BIC if criteria=2, AIC if criteria=1@

eps=0.15;	@chose a value from {0.01, 0.05, 0.10, 0.15, 0.25}@

T=rows(y);@number of sample@

kmax=int(12*(T/100)^(1/4)); @maximum number of lag length@

/*****************************************/
/*Please do not change the following code*/
/*****************************************/
@define some matrix@
if (model==1);
    VR=0~0~1;
	v_t=-4.30|-4.39|-4.39|-4.34|-4.32|-4.45|-4.42|-4.33|-4.27|-4.27;
    cv=1.60~2.07~3.33|1.52~1.97~3.24|1.41~1.88~3.05|1.26~1.74~3.12|0.91~1.33~2.83;
elseif (model==2);
    VR=0~0~1;
	v_t=-4.27|-4.41|-4.51|-4.55|-4.56|-4.57|-4.51|-4.38|-4.26|-4.26;
    cv=1.52~2.02~3.37|1.40~1.93~3.27|1.28~1.86~3.20|1.13~1.67~3.06|0.74~1.28~2.61;
elseif (model==3);
    VR=0~1~0~0|0~0~0~1; 
	v_t=-4.38|-4.65|-4.78|-4.81|-4.90|-4.88|-4.75|-4.70|-4.41|-4.41;
    cv=2.96~3.55~5.02|2.82~3.36~4.78|2.65~3.16~4.59|2.48~3.12~4.47|2.15~2.79~4.57;
endif;
constant=ones(T,1);
trend=seqa(1,1,T);
vect1=zeros(int((1-2*eps)*T)+2,1);


TBi=maxc(int(eps*T)|(kmax+2));
do while TBi<=int((1-eps)*T);

	lam1=TBi/T;
    DUi=(trend.>TBi); DTi=(trend.>TBi).*(trend-TBi); 
    if (model==1);
  		reg=constant~trend~DUi;
    elseif (model==2);
    	reg=constant~trend~DTi;
    elseif (model==3);
    	reg=constant~DUi~trend~DTi;
    endif;
		
	@---Estimation of alpha----@			
	{khat}=IC(y,reg,kmax,criteria);	
	u=(eye(T)-reg*inv(reg'reg)*reg')*y; du=diff(u,1);
	depu=u; regu=lagn(u,1);
	j=1;
	do while j<=khat-1;
		regu=regu~lagn(du,j);
		j=j+1;
	endo;
	depu=trimr(depu,khat,0);
	regu=trimr(regu,khat,0);
	b=inv(regu'regu)*(regu'depu);
	ehat=depu-regu*b;
	VCV=(ehat'ehat/rows(ehat))*inv(regu'regu);
	ahat=b[1]; vahat=VCV[1,1];
	tau1=(ahat-1)/sqrt(vahat);

	@Upper Biased Estimator@
	if lam1<=0.1;                     t05=v_t[1];
	elseif (lam1>0.1)*(lam1<=0.2)==1; t05=v_t[2];
	elseif (lam1>0.2)*(lam1<=0.3)==1; t05=v_t[3];
	elseif (lam1>0.3)*(lam1<=0.4)==1; t05=v_t[4];
	elseif (lam1>0.4)*(lam1<=0.5)==1; t05=v_t[5];
	elseif (lam1>0.5)*(lam1<=0.6)==1; t05=v_t[6];
	elseif (lam1>0.6)*(lam1<=0.7)==1; t05=v_t[7];
	elseif (lam1>0.7)*(lam1<=0.8)==1; t05=v_t[8];
	elseif (lam1>0.8)*(lam1<=0.9)==1; t05=v_t[9];
	elseif lam1>0.9;                  t05=v_t[10];
	endif;
	IP=int((khat+1)/2); r=cols(reg); k=10; c1=sqrt((1+r)*T);
	c2=((r+1)*T-t05*t05*(IP+T))/(t05*(t05+k)*(IP+T));
	if (tau1>t05);
		ctau=-tau1; 
	elseif (tau1<=t05)*(tau1>-k)==1;
		ctau=IP*(tau1/T)-(r+1)/(tau1+c2*(tau1+k));
	elseif (tau1<=-k)*(tau1>-c1)==1;
		ctau=IP*(tau1/T)-(r+1)/tau1;
	elseif (tau1<=-c1);
		ctau=0;
	endif;
	rhomd1 = ahat+ctau*sqrt(vahat);
	rhomd1= 1*(rhomd1>=1)+rhomd1*(abs(rhomd1)<1)-0.99*(rhomd1<=-1);
	amu=rhomd1;

	CR=(T^0.5)*abs(amu-1);	
	amus=amu*(CR>1)+1*(CR<=1);
	gdep=((y[1])~(trimr(y,1,0)-amus*trimr(y,0,1))')';
	greg=((reg[1,.])'~(trimr(reg,1,0)-amus*trimr(reg,0,1))')';	
	b=inv(greg'greg)*greg'gdep;
	v=gdep-greg*b;

	if khat==1;
		h0=v'v/(rows(v));
	else;
		ki=1;
		if (amus==1);
			do while ki<=khat-1;
				if (ki==1);
					regv=lagn(v,ki);
				else;
					regv=regv~lagn(v,ki);
				endif;
				ki=ki+1;
			endo;			
			depv=trimr(v,khat-1,0);
			regv=trimr(regv,khat-1,0);		
			beta=inv(regv'regv)*regv'depv;
			e=depv-regv*beta;

            if model==1;
				vbeta=zeros(3,khat-1);
				ki=1;
				do while ki<=khat-1;
					DUki=(trend.>TBi-ki);
					regki=constant~trend~DUki; 
					gdepki=((y[1])~(trimr(y,1,0)-amus*trimr(y,0,1))')';
					gregki=((reg[1,.])'~(trimr(regki,1,0)-amus*trimr(regki,0,1))')';	
					vbeta[.,ki]=inv(gregki'gregki)*gregki'gdepki;
					ki=ki+1;
				endo;			
				b[3]=b[3]-vbeta[3,.]*beta;
                h0=e'e/(T-khat);
            elseif (model==2);
    			h0=(e'e/(T-khat))/((1-beta'ones(khat-1,1))^2);			
            elseif (model==3);
				vbeta=zeros(4,khat-1);
				ki=1;
				do while ki<=khat-1;
					DUki=(trend.>TBi-ki);
					DTki=(trend.>TBi-ki).*(trend-TBi);
					regki=constant~DUki~trend~DTki; 
					gdepki=((y[1])~(trimr(y,1,0)-amus*trimr(y,0,1))')';
					gregki=((reg[1,.])'~(trimr(regki,1,0)-amus*trimr(regki,0,1))')';	
					vbeta[.,ki]=inv(gregki'gregki)*gregki'gdepki;
					ki=ki+1;
				endo;
                sige=e'e/(T-khat);
    			h0=sige/((1-beta'ones(khat-1,1))^2);			
				b[2]=sqrt(h0)*(b[2]-vbeta[2,.]*beta)/sqrt(sige);
            endif;   
		elseif (abs(amus)<1);
			{m,h0} = h0W(v);
		endif;
	endif;
	VCV=h0*inv(greg'greg);
	vect1[TBi-int(eps*T)+1]=(VR*b)'*inv(VR*VCV*VR')*(VR*b);

	TBi=TBi+1;
endo;

wald = ln(sumc(exp(vect1./2))/T);
TB = breakdate(y,model,kmax,eps);


print "*********************************************";
if (model==1);
    print "Model 1: Structural change in intercept";
    print "Y{t}=a0+a1*DU+b0*t+e{t} where DU=1(t>TB)";
elseif (model==2);
    print "Model 2: Structural change in slope";
    print "Y{t}=a0+b0*t+b1*DT+e{t} where DT=1(t>TB)*(t-TB)";
elseif (model==3);
    print "Model 3: Structural change in both intercept and slope";
    print "Y{t}=a0+a1*DU+b0*t+b1*DT+e{t} where DU=1(t>TB) and DT=1(t>TB)*(t-TB)";
endif;
print "";
print "THe EXP test Statistic (W-RQF)  =" wald;				
print " The critical values are the following:";
print "   10%     5%      1%";
    if eps==0.01; cv[1,.];
    elseif eps==0.05; cv[2,.];
    elseif eps==0.10; cv[3,.];
    elseif eps==0.15; cv[4,.];
    elseif eps==0.25; cv[5,.]; endif;
print "The estimate of the break date is TB=" TB;
print "obtained by minimizing the sum of squared residuals from a regression of the
relevant series on a constant, a time trend,";
if (model==1);
    print "a level shift dummy." ;
elseif (model==2);
    print "a slope shift dummy." ;
elseif (model==3);
    print "a level shift and a slope shift dummies." ;
endif;
print "*********************************************";





/*
to take the n lag of x.
x is a TxK matrix. return is also a TxK matrix with the first n elements 0.
*/  
proc(1) = lagn(x,n); 
	local y;
	if n>0;
		y = zeros(n,cols(x)) | trimr(x,0,n);
	else;
		y = trimr(x,abs(n),0) | zeros(abs(n),cols(x));
	endif;
	retp(y);
endp;

/*
diff(x,k) is same as x{t}-x{t-k}
*/  
proc(1) = diff(x,k); 
	local y;
	if(k==0);
		y = x;
	else;
		y = zeros(k,cols(x)) | (trimr(x,k,0)-trimr(lagn(x,k),k,0));
	endif;
	retp(y);
endp;

/*
AIC if criteria=1, BIC if criteria=2
*/
proc(1) = IC(x,reg,kmax,criteria);
	local ICV, dep, rego, khat, b, e, fit, i;
	ICV = zeros(kmax+1,1);
	dep = trimr(x,kmax,0);
	rego = trimr(reg,kmax,0);
	e=dep-rego*inv(rego'rego)*rego'*dep;
	ICV[1] = ln(e'e/rows(e)); /* in the case of k=0.*/
	i = 1;
	do while i <= kmax;
		reg = reg~lagn(x,i);
		rego = trimr(reg,kmax,0);
		{b,e,fit} = olsqr2(dep,rego);
		if (criteria==1);@AIC@
			ICV[i+1] = ln(e'e/rows(e))+(2*i)/(rows(e));		
		elseif (criteria==2);@BIC@
			ICV[i+1] = ln(e'e/rows(e))+(ln(rows(e))*i)/(rows(e));
		endif;
		i = i+1;
	endo;
	khat = minindc(ICV)-1;
	khat=khat*(khat>=1)+1*(khat==0);
	retp(khat);
endp;

/*
This function is to take the autocovariance functions of x as
ACF(x) = [R(0), R(1),...,R(T-1)]'.
Thus, R[1]=R(0), R[2]=R(1), and so on.
*/ 
proc(1) = ACV(x);
	local R, rho, i, j, xbar, T;
	T = rows(x);
	R = zeros(T,1); 
	xbar = meanc(x); 
	i = 1;
	do while i <= T;
		j = i-1;
		R[i] = (x[1:T-j,1]-xbar)'(x[1+j:T,1]-xbar)/T;
		i = i+1;	
	endo; 
	retp(R);
endp;


@---h(0)---@
/*
Andrews'(1991) method using a window:
Quadratic Spectral window
*/
proc(2) = h0W(x);
	local m,s,R,a,delta,lamda,h0,i,b,ll,e,vcb,IC1,IC2, T;

    T=rows(x);	
    b=olsqr(x[2:T],x[1:T-1]);
	a=(4*(b^2))/((1-b)^4);
	R = ACV(x);
	lamda = zeros(T-1,1);
	s=seqa(1,1,T-1);

	m = 1.3221*((a*T)^(1/5));
	delta = (6*pi*s)/(5*m);
	i=1;
	do while i<=T-1; 
		lamda[i] = 3*((sin(delta[i])/delta[i])-cos(delta[i]))/(delta[i]^2);
		i = i+1;
	endo;

	h0 = (R[1] + 2*lamda'*R[2:T,1]);

	retp(m,h0);	
endp;

proc(1) = lagn(x,n); 
	local y;
	if n>0;
		y = zeros(n,cols(x)) | trimr(x,0,n);
	else;
		y = trimr(x,abs(n),0) | zeros(abs(n),cols(x));
	endif;
	retp(y);
endp;


proc(1) = breakdate(x,model,kmax,eps);
	local vsum, T, TB, constant, trend, TB0, TBN, TBi, date, DUi, DTi, reg, e;

    T=rows(x);
    constant=ones(T,1); 
    trend=seqa(1,1,T);
    TB0=maxc(int(eps*T)|(kmax+2));
    TBN=int((1-eps)*T);

    vsum=ones(TBN-TB0+1,1)*1000;
    TBi=TB0;
    do while TBi<=TBN;
        DUi=(trend.>TBi); DTi=(trend.>TBi).*(trend-TBi); 
        if (model==1);
  		    reg=constant~trend~DUi;
        elseif (model==2);
    	    reg=constant~trend~DTi;
        elseif (model==3);
    	    reg=constant~DUi~trend~DTi;
        endif;
		e=(eye(T)-reg*inv(reg'reg)*reg')*x;
        vsum[TBi-TB0+1]=e'e;
        TBi=TBi+1;
    endo;

    TB=TB0+minindc(vsum)-1;
    retp(TB);	
endp;